
local _, class = UnitClass("player")
if class ~= "DEATHKNIGHT" then
	return
end
local debug = false

local plugin = mRunes:NewModule("Cooldowns", "AceEvent-3.0")
plugin.mRunesPlugin = true

local AceGUI = LibStub("AceGUI-3.0")
local LSM = LibStub("LibSharedMedia-3.0")
local BD = LibStub("LibBackdrop-1.0")
local FA = LibStub("LibFrameAnchorRegistry-1.0")
local UIF
local db

local backdrop = {
	tile = true,
	tileSize = 16,
	insets = {},
}

local spells = {
	-- ALL --
	-- BLOOD --
	-- FROST --
	{ id = "PillarOfFrost", spell = 51271, spec = 2 },
	{ id = "EmpowerRuneWeapon", spell = 47568, spec = 2, talent = { spell = 207127, spec = 2, tier = 3, column = 2}},
	{ id = "HornOfWinter", spell = 57330, spec = 2, talent = { spec = 2, tier = 2, column = 3}},
	{ id = "BreathOfSindragosa", spell = 152279, spec = 2, talent = { spec = 2, tier = 7, column = 2}},

	-- UNHOLY --
	{ id = "DarkTransformation", spell = 63560, spec = 3 },
	{ id = "SoulReaper", spell = 130736, spec = 3, talent = { spec = 3, tier = 7, column = 3} },
	{ id = "DeathAndDecay", spell = 43265, spec = 3, talent = { spell = 152280, spec = 3, tier = 7, column = 2} },
	{ id = "MindFreeze", spell = 47528 },
	{ id = "SummonGargoyle", spell = 49206, spec = 3, talent = { spell = 207349, spec = 3, tier = 7, column = 1} },
	{ id = "Outbreak", spell = 77575, spec = 3 },
	{ id = "BlightedRuneWeapon", spell = 194918, spec = 3, talent = { spec = 3, tier = 2, column = 3} },

	--{ id = "Necrosis", spell = 207346, spec = 3, talent = { spec = 3, tier = 6, column = 2} }, Self check for buff
	--{ id = "FesteringWounds", spell = 194310, spec = 3 }, Need to check target, and also show stacks
}

local cooldowns = {}

plugin.DisplayName = "Cooldowns"
plugin.DisplayIcon = [[Interface\Icons\spell_deathknight_bloodplague]]
plugin.DisplayOrder = 4

function plugin:OnInitialize()
	local defaults = {
		profile = {
			Enabled = true,
			Attached = {
				f = "mRunesRunes",
				x = 0,
				y = 0,
				Point = "BOTTOMLEFT",
				RelPoint = "TOPLEFT"
			},
			Height = 200,
			Width = 200,

            Font = "accid",
            FontSize = 16,
            FontFlags = "OUTLINE",
            FontColor = {1, 1, 1, 1},
            HorAlign = "LEFT",
            InternalSpacing = 2,


			BorderTexture = "None",
			BorderWidth = 5,
			BorderInsets = 0,
			BorderColor = {0, 0, 0, 1},
			BorderPadding = 2,
			BackgroundColor = {0, 0, 0, 0.5},
			CooldownsAlphaInCombat = 1,
			--CooldownsAlphaOutCombat = 1,
			CooldownsAlphaOutCombat = 0.2
		}
	}

	self.db = mRunes.db:RegisterNamespace(self:GetName(), defaults)

	self.db  = LibStub("AceDB-3.0"):New("mRunesDB", defaults, "Default")
	self.db.RegisterCallback(self, "OnProfileChanged", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileCopied", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileReset", "OnProfileChanged")
	db = self.db.profile

	self:SetEnabledState(db.Enabled)
	UIF = LibStub("LibGUIFactory-1.0"):GetFactory("mRunes")

	if not self.CooldownFrame then
		self:CreateCooldownFrame()
	end
end

function plugin:OnEnable()
	self:ApplySettings()

	self:RegisterEvent("PLAYER_TALENT_UPDATE", "OnTalentUpdate")
	self:RegisterEvent("SPELL_UPDATE_COOLDOWN", "OnCooldown")
	self:RegisterEvent("PLAYER_ENTERING_WORLD", "OnCooldown")

	mRunes.RegisterCallback(self, "EnteringCombat")
	mRunes.RegisterCallback(self, "LeavingCombat")


	self.CooldownFrame:Show()
	mRunes:RegisterOnUpdate(self, self.OnUpdate)

end

function plugin:OnDisable()
	self.CooldownFrame:Hide()
	mRunes:UnregisterOnUpdate(self, self.OnUpdate)
	mRunes.UnregisterAllCallbacks(self)
end

function plugin:OnUpdate()

end

function plugin:OnProfileChanged()
	db = self.db.profile
	self:ApplySettings()
end

function plugin:OnTalentUpdate(...)


	plugin:ReAnchor()
end

function plugin:OnCooldown(a, b, c, d)
	for i, button in pairs(cooldowns) do
		plugin:StartCooldown(button, button.spellId)
	end
end

function plugin:EnteringCombat(event)
	mRunes.FadeIn(self.CooldownFrame, false, db.CooldownsAlphaInCombat)
	InCombat = true
end

function plugin:LeavingCombat(event)
	mRunes.FadeOut(self.CooldownFrame, db.CooldownsAlphaOutCombat)
	InCombat = false
end

function plugin:ApplySettings()

	local cooldownFrame = self.CooldownFrame
	if cooldownFrame then

		local anchor = FA:GetAnchor(db.Attached.f)
		cooldownFrame:ClearAllPoints()
		cooldownFrame:SetPoint(db.Attached.Point, anchor, db.Attached.RelPoint, db.Attached.x, db.Attached.y)
		cooldownFrame:SetWidth(db.Width)
		cooldownFrame:SetHeight(db.Height)

		backdrop.insets.left = db.BorderInsets
		backdrop.insets.right = db.BorderInsets
		backdrop.insets.top = db.BorderInsets
		backdrop.insets.bottom = db.BorderInsets
		-- backdrop.edgeFile = LSM:Fetch("border", db.BorderTexture)
		-- backdrop.edgeSize = db.BorderWidth
		-- backdrop.bgFile = LSM:Fetch("background", "Solid")
		-- cooldownFrame:SetBackdrop(backdrop)
		-- cooldownFrame:SetBackdropColor(unpack(db.BackgroundColor))
		-- cooldownFrame:SetBackdropBorderColor(unpack(db.BorderColor))

		cooldownFrame:SetAlpha(
			(InCombat and db.CooldownsAlphaInCombat) or db.CooldownsAlphaOutCombat)
		if db.Enabled then cooldownFrame:Show() else cooldownFrame:Hide() end
	end

	self:ReAnchor()

	if not db.Enabled and self:IsEnabled() then self:Disable()
	elseif db.Enabled and not self:IsEnabled() then self:Enable() end
end

function plugin:ReAnchor()
dprint("--------- START OCH ReAnchor ---------")
	for i, button in pairs(cooldowns) do
		button:Hide()
		if button.talent and button.talent.spell and self:AvalibleTalent(button.talent) then
			button.spellId = button.talentedSpellId
		else
			button.spellId = button.untalentedSpellId
		end

		local spellIcon = GetSpellTexture(button.untalentedSpellId)
		button.icon:SetTexture(spellIcon)

		dprint(button.spellName, button.spellId, button.untalentedSpellId, button.talentedSpellId)
	end


	local visibleCooldowns = {}
	local spec = GetSpecialization()
	for i, button in pairs(cooldowns) do
		if(not button.talent or self:AvalibleTalent(button.talent) or button.talent.spell) then
			if(not button.spec or button.spec == spec) then
				tinsert(visibleCooldowns, button)
			end
		end
	end

	for i, button in pairs(visibleCooldowns) do

		local width = 32
		local height = 32
		local offX = (i-1) * (width + 10)
		local offY = 0
		button:SetPoint("BOTTOMLEFT", offX, offY)
		button:SetSize(width, height)
		button:Show()

		-- local spellName, _, spellIcon = GetSpellInfo(button.spellId)
		-- dprint(spellName)

		--button.icon:SetTexture(spellIcon)

	end
	dprint ("end of ReAnchor")
end



function plugin:CreateCooldownFrame()
	dprint("create")
	local frame = CreateFrame("Frame", nil, UIParent)

	FA:RegisterAnchor("mRunes", "Cooldowns", "mRunesCooldowns", frame)

	local count = 1
	for i, data in pairs(spells) do
		local spellName, _, spellIcon = GetSpellInfo(data.spell)

		if spellName then
			local button = self:CreateButton(frame, data.id, spellName)
			button.spellId = data.spell
			button.untalentedSpellId = data.spell
			button.talentedSpellId = data.talent and data.talent.spell
			button.spellName = spellName
			button.spellIcon = spellIcon
			button.id = data.id
			button.spec = data.spec
			button.talent = data.talent

			cooldowns[count] = button
			count = count + 1

			dprint(button.spellName, button.spellId, button.untalentedSpellId, button.talentedSpellId)
		end
	end

	self.CooldownFrame = frame
	--self:ApplySettings()
end

function plugin:CreateButton(parent, buttonName, spellName)

	local frame = CreateFrame("Button", "mRunes\001Cooldowns"..buttonName.."CooldownBackground", parent)
	local icon = frame:CreateTexture(nil, "ARTOWORK")
	local cooldown = CreateFrame("Cooldown", "mRunes"..spellName.."Cooldown", frame, "CooldownFrameTemplate")


	frame.icon = icon
	frame.cooldown = cooldown

	frame.icon:SetAllPoints()
	frame.icon:Show()
	frame.cooldown:SetAllPoints()

	FA:RegisterAnchor("mRunes\001Cooldowns", spellName, "mRunesCooldowns"..buttonName, frame)

	return frame
end

function plugin:AvalibleTalent(talent)
		if not talent then return false end

		local spec = GetSpecialization()
		local talentID, name, texture, selected, available = GetTalentInfo(talent.tier, talent.column, GetActiveSpecGroup())

		return spec == talent.spec and selected
end

function plugin:StartCooldown(frame, spellId)
	local start, duration = GetSpellCooldown(frame.spellId)
	if start > 0 and duration > 0 then
		if spellId == 43265 or spellId == 152280 then
			dprint(frame.spellName)
			dprint(frame.spellId)
		end
		frame.cooldown:SetCooldown(start, duration)
	end
end

-- Options

local lastCooldownSelected
local cooldownSelected
local cooldownTabs = {}

local function Callback()
	plugin:ApplySettings()
end

function plugin:GetOptions()
	return plugin.GetGUI
end

function plugin:GetGUI()
	self = plugin
	local returnGroup = AceGUI:Create("SimpleGroup")
	returnGroup:AddChild(UIF:CheckBox("Enabled", db, "Enabled", Callback, nil, 0.33))

	return returnGroup
end


cooldownSelected = function(self, event, tab)

end

GlobalSettings = function(self)

end

function dprint(...)
	if(debug) then print(...) end
end