local _, class = UnitClass("player")
if class ~= "DEATHKNIGHT" then
	return
end

local plugin = mRunes:NewModule("Runic Power", "AceEvent-3.0")
plugin.mRunesPlugin = true

local AceGUI 	= LibStub("AceGUI-3.0")
local LSM 		= LibStub("LibSharedMedia-3.0")
local BD 		= LibStub("LibBackdrop-1.0")
local FA		= LibStub("LibFrameAnchorRegistry-1.0")
local UIF
local db

local InCombat = false
local HaveRP = false
local ConfigMode = false

local backdrop = {
	tile = true, 
	tileSize = 16,
	insets = {},
}

plugin.DisplayName = "Runic Power"
plugin.DisplayIcon = [[Interface\Icons\Spell_DeathKnight_DarkConviction]]
plugin.DisplayOrder = 2

function plugin:OnInitialize()
	local defaults = {
		profile = {
			Enabled = true,
			
			Attached = {
				f = "mRunesRunes",
				x = 0,
				y = 0,
				Point = "TOP",
				RelPoint = "BOTTOM",
			},
			
			Width = 330,
			Height = 20,			
			InternalSpacing = 1,						
			AlphaIncCombat = 1,
			AlphaOutCombat = 0.2,
			AlphaActive = 0.5,
			
			HorAlign = "LEFT",
			AlwaysShowText = false,
			ShowText = true,
			
			Font = "accid",
			FontSize = 16,
			FontFlags = "OUTLINE",
			FontColor = {1, 1, 1, 1},
			
			Texture = "HalD",
			PowerColor = {0, 0.82, 1, 1},
			
			BorderTexture = "None",
			BorderWidth = 5,
			BorderInsets = 0,
			BorderColor = {0, 0, 0, 1},
			BorderPadding = 2,
			BackgroundColor = {0, 0, 0, 0.5},
		}
	}
	
	self.db = mRunes.db:RegisterNamespace(self:GetName(), defaults)
	self.db.RegisterCallback(self, "OnProfileChanged", "ProfileChanged")
	self.db.RegisterCallback(self, "OnProfileCopied", "ProfileChanged")
	self.db.RegisterCallback(self, "OnProfileReset", "ProfileChanged")
	db = self.db.profile
	
	self:SetEnabledState(db.Enabled)
	
	UIF = LibStub("LibGUIFactory-1.0"):GetFactory("mRunes")
	
	if not self.PowerFrame then
		self:CreatePowerFrame()
	end	
end

function plugin:OnEnable()		
	self:ApplySettings()
	
	self:RegisterEvent("UNIT_MAXRUNIC_POWER", "RunicPowerChange")
	self:RegisterEvent("UNIT_RUNIC_POWER", "RunicPowerChange")
	self:RegisterEvent("UNIT_POWER", "RunicPowerChange")
	self:RegisterEvent("UNIT_MAXPOWER", "RunicPowerChange")
	
	mRunes.RegisterCallback(self, "EnteringCombat")
	mRunes.RegisterCallback(self, "LeavingCombat")
	mRunes.RegisterCallback(self, "ToggleConfigMode")
	
	if UnitAffectingCombat("player") then
		self:EnteringCombat("EnteringCombat")
	else
		self:LeavingCombat("LeavingCombat")
	end
	
	mRunes:RegisterOnUpdate(self.PowerFrame, self.RunicPowerChange, 0.1)
end

function plugin:OnDisable()
	mRunes:UnregisterOnUpdate(self.PowerFrame, self.RunicPowerChange)
	mRunes.UnregisterAllCallbacks(self)
	if self.PowerFrame then mRunes.FadeOut(self.PowerFrame) end
end

function plugin:ProfileChanged()
	db = self.db.profile
	self:ApplySettings()
end

-- ~~~~~~~~~~~~~~~~~~~~
-- Callbacks ~~~~~~~~~~
-- ~~~~~~~~~~~~~~~~~~~~
function plugin:EnteringCombat(event)
	mRunes.FadeIn(self.PowerFrame, false, db.AlphaIncCombat)
	InCombat = true
	HaveRP = false
end

function plugin:LeavingCombat(event)
	if UnitPower("player") <= 0 then mRunes.FadeOut(self.PowerFrame, db.AlphaOutCombat) end
	InCombat = false
end

function plugin:EnterWorld(...)
end

function plugin:ToggleConfigMode(event, enable)
	ConfigMode = enable
	if enable then
		self.PowerFrame.text:Show()
		self.PowerFrame.ConfigText:Show()
		mRunes.FadeIn(self.PowerFrame, false, 1)
		self.PowerFrame.bar:SetValue(0.7)		
		self.PowerFrame.text:SetText(75)
	else
		self.PowerFrame.text:Hide()
		self.PowerFrame.ConfigText:Hide()
		self.PowerFrame.bar:SetValue(0)
		self.PowerFrame.text:SetText()
		if InCombat then
			mRunes.FadeIn(self.PowerFrame, true, db.AlphaIncCombat)
		elseif HaveRP then
			mRunes.FadeIn(self.PowerFrame, true, db.AlphaActive)
		else
			mRunes.FadeIn(self.PowerFrame, true, db.AlphaOutCombat)
		end
	end
end

-- ~~~~~~~~~~~~~~~~~~~~
-- Events ~~~~~~~~~~~~~
-- ~~~~~~~~~~~~~~~~~~~~
function plugin:RunicPowerChange(event, unit)
	if ConfigMode then return end
	self = plugin
	if unit and unit ~= "player" then return end
	local power = UnitPower("player")
	local maxPower = UnitPowerMax("player")
	local p = power / maxPower	
	local haveRP = false
	local oldHaveRP = HaveRP
	if power > 0 then
		haveRP = true
		self.PowerFrame.bar:SetValue(p)		
		self.PowerFrame.text:SetFormattedText("%d", power)
		if db.ShowText and not self.PowerFrame.text:IsShown() then self.PowerFrame.text:Show() end
	else
		mRunes:UnregisterOnUpdate(self.PowerFrame, self.RunicPowerChange)
		self.PowerFrame.bar:SetValue(0)
		self.PowerFrame.text:SetFormattedText("%d", 0)
		if not db.AlwaysShowText and not ConfigMode then 
			if self.PowerFrame.text:IsShown() then self.PowerFrame.text:Hide() end
		elseif db.ShowText then
			if not self.PowerFrame.text:IsShown() then self.PowerFrame.text:Show() end
		else
			if self.PowerFrame.text:IsShown() then self.PowerFrame.text:Hide() end
		end		
	end
	
	if not InCombat then
		HaveRP = haveRP
		if HaveRP and (HaveRP ~= oldHaveRP) then
			local currAlpha = self.PowerFrame:GetAlpha()
			if currAlpha < db.AlphaActive then
				mRunes.FadeIn(self.PowerFrame, false, db.AlphaActive)				
			else
				mRunes.FadeOut(self.PowerFrame, db.AlphaActive)
				local t = 2
			end
		elseif not HaveRP and (HaveRP ~= oldHaveRP) then
			mRunes.FadeOut(self.PowerFrame, db.AlphaOutCombat)
		end
	end
end

-- ~~~~~~~~~~~~~~~~~~~~
-- Settings	~~~~~~~~~~~
-- ~~~~~~~~~~~~~~~~~~~~
function plugin:ApplySettings()
	local PowerFrame = self.PowerFrame
	if PowerFrame then
		local anchor = FA:GetAnchor(db.Attached.f)
		PowerFrame:ClearAllPoints()
		PowerFrame:SetPoint(db.Attached.Point, anchor, db.Attached.RelPoint, db.Attached.x, db.Attached.y)
		PowerFrame:SetWidth(db.Width)
		PowerFrame:SetHeight(db.Height)
		
		backdrop.insets.left = db.BorderInsets
		backdrop.insets.right = db.BorderInsets
		backdrop.insets.top = db.BorderInsets
		backdrop.insets.bottom = db.BorderInsets
		backdrop.edgeFile = LSM:Fetch("border", db.BorderTexture)
		backdrop.edgeSize = db.BorderWidth
		backdrop.bgFile = LSM:Fetch("background", "Solid")
		PowerFrame:SetBackdrop(backdrop)
		PowerFrame:SetBackdropColor(unpack(db.BackgroundColor))
		PowerFrame:SetBackdropBorderColor(unpack(db.BorderColor))
		
		local leftinset, rightinset = 0, 0
		if db.HorAlign == "LEFT" then
			leftinset = db.InternalSpacing
		elseif db.HorAlign == "RIGHT" then
			rightinset = -db.InternalSpacing
		end	
		
		PowerFrame.text:SetPoint("TOPLEFT", leftinset, 0)
		PowerFrame.text:SetPoint("BOTTOMRIGHT", rightinset, 0)
		PowerFrame.text:SetFont(LSM:Fetch("font", db.Font), db.FontSize, db.FontFlags)
		PowerFrame.text:SetTextColor(unpack(db.FontColor))
		PowerFrame.text:SetJustifyH(db.HorAlign)
		PowerFrame.text:SetJustifyV("MIDDLE")
		
		local bar = PowerFrame.bar
		bar:ClearAllPoints()
		bar:SetPoint("TOPLEFT", db.BorderPadding, -db.BorderPadding)
		bar:SetPoint("BOTTOMRIGHT", -db.BorderPadding, db.BorderPadding)
		
		bar:SetStatusBarTexture(LSM:Fetch("statusbar", db.Texture))
		bar:GetStatusBarTexture():SetHorizTile(false)
		bar:SetStatusBarColor(unpack(db.PowerColor))
		
		if not ConfigMode then
			PowerFrame:SetAlpha(InCombat and db.AlphaIncCombat or (HaveRP and db.AlphaActive or db.AlphaOutCombat))
		end
		if PowerFrame:GetAlpha() > 0 then PowerFrame:Show() else PowerFrame:Hide() end		
		if db.AlwaysShowText and db.ShowText then
			PowerFrame.text:Show()
		elseif not db.ShowText then
			PowerFrame.text:Hide()				
		end
		
		self:RunicPowerChange("UNIT_RUNIC_POWER", "player")
	end	
	
	if not db.Enabled and self:IsEnabled() then self:Disable()
	elseif db.Enabled and not self:IsEnabled() then self:Enable() end
end

-- ~~~~~~~~~~~~~~~~~~~~
-- Create Frames ~~~~~~
-- ~~~~~~~~~~~~~~~~~~~~
function plugin:CreatePowerFrame()
	local frame = CreateFrame("Frame", nil, UIParent)
	
	--BD:EnhanceBackdrop(frame)
	FA:RegisterAnchor("mRunes", "Runic Power", "mRunesRunicPower", frame)
	
	local bar = CreateFrame("StatusBar", nil, frame)
	bar:SetMinMaxValues(0,1)
	frame.bar = bar
	
	local text = bar:CreateFontString(nil, nil, "GameFontNormal")
	frame.text = text	
	
	local configFrame = CreateFrame("Frame", nil, frame)
	configFrame:SetAllPoints()
	configFrame:SetFrameLevel(bar:GetFrameLevel() + 10)
	local configText = configFrame:CreateFontString(nil, "OVERLAY", "GameFontNormal")
	configText:SetAllPoints()
	configText:SetText("Runic Power")
	configText:Hide()
	frame.ConfigText = configText
	
	self.PowerFrame = frame
end

-- ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-- Options ~~~~~~~~~~~~~~~~~~~~
-- ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
function plugin:GetOptions()
	return plugin.GetGUI
end

do
	local function Callback()
		plugin:ApplySettings()
	end
	
	local FontFlagList = {
		["NONE"] = "None",
		["OUTLINE"] = "Outline",
		["THICKOUTLINE"] = "Thick Outline",
		["MONOCHROME"] = "Monochrome",
	}
	
	local VerticalAlignList = {
		["TOP"] = "Top",
		["MIDDLE"] = "Middle",
		["BOTTOM"] = "Bottom",
	}
	
	local HorizontalAlignList = {
		["LEFT"] = "Left",
		["CENTER"] = "Center",
		["RIGHT"] = "Right",
	}
	
	local TabSelected
	local LastTablSelected

	
	function plugin:GetGUI()
		self = plugin
		
		local returnGroup = AceGUI:Create("SimpleGroup")
		returnGroup:SetFullWidth(true)
		returnGroup:SetLayout("Flow")
		
		returnGroup:AddChild(UIF:NewLine(10))
		
		returnGroup:AddChild(UIF:CheckBox("Enabled", db, "Enabled", Callback, nil, 0.33))
		
		local tgroup = UIF:TabGroup()
		returnGroup:AddChild(tgroup)
		tgroup:SetTabs({
			{text = "Position & Size", value = "PaS"},
			{text = "Colors & Alpha", value = "CaA"},
			{text = "Font", value = "Font"},
		})		
		tgroup:SetCallback("OnGroupSelected", TabSelected)
		tgroup:SelectTab(LastTablSelected or "PaS")
		
		return returnGroup
	end
	
	TabSelected = function(self, event, tab)
		self:ReleaseChildren()
		LastTablSelected = tab
		
		if tab == "PaS" then
		
			self:AddChild(UIF:Text1("Select a frame to attach to. You can then adjust the relative positioning to that frame via X and Y as well as the anchor points."))			
			local t = AceGUI:Create("FrameAnchorDropdown")
			t:SetRelativeWidth(0.5)
			t:SetList(FA:GetAnchors(plugin.PowerFrame))
			t:SetLabel("Anchor")
			t:SetValue(db.Attached.f)
			t:SetCallback("OnValueChanged", function(self, event, cat, name, frame)
				db.Attached.f = cat
				Callback()
			end)
			self:AddChild(t)
			self:AddChild(UIF:Spacer(10))
			self:AddChild(UIF:Text2("Avoid creating circle references!", nil, nil, nil, nil, nil, 0.4))
			self:AddChild(UIF:NewLine())			
			self:AddChild(UIF:Dropdown("Point", db.Attached, "Point", Callback, mRunes:GetAnchorList(), 0.5, true))
			self:AddChild(UIF:Dropdown("Relative Point", db.Attached, "RelPoint", Callback, mRunes:GetAnchorList(), 0.5, true))
			self:AddChild(UIF:Slider("X", db.Attached, "x", -300, 300, 0.1, Callback, 0.5))
			self:AddChild(UIF:Slider("Y", db.Attached, "y", -300, 300, 0.1, Callback, 0.5))
			self:AddChild(UIF:Slider("Width", db, "Width", 5, 1000, 0.1, Callback, 0.5))
			self:AddChild(UIF:Slider("Height", db, "Height", 5, 200, 0.1, Callback, 0.5))		
		
		elseif tab == "CaA" then
			local group = UIF:InlineGroup2("Bar Color")
			self:AddChild(group)
			group:AddChild(UIF:ColorSelect("Runic Power color", db, "PowerColor", Callback, true, 1.0))		
			
			group = UIF:InlineGroup2("Alpha")			
			self:AddChild(group)
			group:AddChild(UIF:Slider("In combat", db, "AlphaIncCombat", 0, 1, 0.01, Callback, 0.33))
			group:AddChild(UIF:Slider("OOC (RP > 0)", db, "AlphaActive", 0, 1, 0.01, Callback, 0.33))
			group:AddChild(UIF:Slider("OOC (RP = 0)", db, "AlphaOutCombat", 0, 1, 0.01, Callback, 0.33))
			
			group = UIF:InlineGroup2("Border and Background")
			self:AddChild(group)
			group:AddChild(UIF:LSMDropdown("border", "Border texture", db, "BorderTexture", Callback, 0.5))
			group:AddChild(UIF:StatusbarSelect("Bar Texture", db, "Texture", Callback, 0.5))				
			
			group:AddChild(UIF:ColorSelect("Background Color", db, "BackgroundColor", Callback, true, 0.5))			
			group:AddChild(UIF:ColorSelect("Border color", db, "BorderColor", Callback, true, 0.5))		
			group:AddChild(UIF:Slider("Border Width", db, "BorderWidth", 0, 50, 1, Callback, 0.33))
			group:AddChild(UIF:Slider("Border Padding", db, "BorderPadding", 0, 100, 0.1, Callback, 0.33))
			group:AddChild(UIF:Slider("Border Insets", db, "BorderInsets", 0, 50, 1, Callback, 0.33))				
		
		elseif tab == "Font" then
			self:AddChild(UIF:Text1("These settings are for the text display of current runic power."))
			self:AddChild(UIF:CheckBox("Show", db, "ShowText", Callback, "Toggle if the runic power text should be shown.", 0.5))
			self:AddChild(UIF:CheckBox("Always Show", db, "AlwaysShowText", Callback, "Toggle if the runic power text should always be shown. If not it will be hidden when the runc power is at 0", 0.5))
			self:AddChild(UIF:LSMDropdown("font", "Font", db, "Font", Callback, 0.5))
			self:AddChild(UIF:Dropdown("Font Flags", db, "FontFlags", Callback, FontFlagList, 0.5))		
			self:AddChild(UIF:ColorSelect("Color", db, "FontColor", Callback, false, 0.2))
			self:AddChild(UIF:Slider("Size", db, "FontSize", 2, 50, 1, Callback, 0.4))
			self:AddChild(UIF:Slider("Margin", db, "InternalSpacing", 0, 50, 0.1, Callback, 0.4))
			self:AddChild(UIF:Dropdown("Horizontal Align", db, "HorAlign", Callback, HorizontalAlignList, 0.5))			
		end
		
		self:DoLayout()
		self.parent:DoLayout()
	end
end